

/*
This is a UI file (.ui.qml) that is intended to be edited in Qt Design Studio only.
It is supposed to be strictly declarative and only uses a subset of QML. If you edit
this file manually, you might introduce QML code that is not supported by Qt Design Studio.
Check out https://doc.qt.io/qtcreator/creator-quick-ui-forms.html for details on .ui.qml files.
*/
import QtQuick
import QtQuick.Controls
import QtQuick3D
import ParticleEffects
import QtQuick3D.Particles3D
import QtQuick3D.Helpers
import Quick3DAssets.Zen_SceneTest 1.0
import QtQml
import QtQuick3D.Effects 6.0

Rectangle {
    id: rectangle1
    width: Constants.width
    height: Constants.height
    color: "#000000"
    border.color: "#000000"
    layer.format: ShaderEffectSource.RGB

    View3D {
        id: view3D
        anchors.fill: parent
        anchors.leftMargin: demoUI.menuphase * demoUI.menuWidth
        environment: sceneEnvironment
        layer.format: ShaderEffectSource.RGB
        property real tv: view3D.renderStats

        SceneEnvironment {
            id: sceneEnvironment
            probeHorizon: 0
            specularAAEnabled: true
            probeExposure: 1
            depthPrePassEnabled: false
            tonemapMode: SceneEnvironment.TonemapModeLinear
            backgroundMode: SceneEnvironment.Color
            antialiasingMode: SceneEnvironment.MSAA
            antialiasingQuality: SceneEnvironment.High
        }

        Node {
            id: scene
            visible: true

            Model {
                id: sky
                source: "#Sphere"
                scale.z: 100
                scale.y: 100
                scale.x: 100
                materials: skyMaterial
            }

            DirectionalLight {
                id: directionalLight
                visible: true
                color: demoUI.altScene ? "#66aaff" : "#fffcee"
                ambientColor.r: color.r * 0.6
                ambientColor.g: color.g * 0.8
                ambientColor.b: color.b * 1.0
                eulerRotation.z: 180
                eulerRotation.y: 134.53545
                eulerRotation.x: -17.51563
                shadowMapQuality: Light.ShadowMapQualityVeryHigh
                shadowFactor: 99
                castsShadow: true
                brightness: 1
            }

            Node {
                id: windSource
                property vector3d direction: Qt.vector3d(0.5, 0, -1)
                property real intensity: 10.0
                property real speed: 3.0
                property real gust: 0.0
                property real noiseResolution: 100.0
            }

            Model {
                id: shockSource
                property vector3d sourceposition: Qt.vector3d(shockSource.x,
                                                              shockSource.y,
                                                              shockSource.z)
                property real sourcescale: 50.0 * phase
                property real intensity: 100.0 * (1.0 - phase) * phase
                property real speed: .10 * (1.0 - phase) * phase
                property real gust: 50.0 * (1.0 - phase) * phase
                property real noiseResolution: 1000.0
                property real length: 5000.0 * phase + (5000.0 * phase)
                scale: Qt.vector3d(sourcescale, sourcescale, sourcescale)
                x: 0
                y: 0
                visible: false
                source: "#Sphere"
                castsReflections: false
                receivesShadows: false
                castsShadows: false
                z: 0
                materials: defaultMaterial
                property real phase: 1.0
                NumberAnimation {
                    id: shockAnimator
                    target: shockSource
                    property: "phase"
                    alwaysRunToEnd: false
                    from: 0
                    to: 1
                    duration: 1500
                    onFinished: shockSource.phase = 1.0
                }
            }

            Node {
                id: cameraRoot
                eulerRotation.z: -0.00001
                eulerRotation.y: -155.10498
                eulerRotation.x: -1.02189

                PerspectiveCamera {
                    id: sceneCamera
                    x: -0
                    y: 95.285
                    z: 530.90942
                }
            }

            Node {
                id: effects

                Node {
                    id: sparkroot
                    y: 40
                    eulerRotation.y: numberAnimation.phase

                    SparksEffect {
                        id: sparks
                        visible: demoUI.mainSelection == "Spark"
                        eulerRotation.z: -90
                        y: 0
                        z: 0

                        Model {
                            id: cube
                            x: 58
                            y: 29
                            visible: false
                            source: "#Cube"
                            z: -11
                            eulerRotation.y: 45
                            eulerRotation.z: 45
                            eulerRotation.x: 45
                            scale.z: 0.1
                            scale.y: 0.01
                            scale.x: 1
                            materials: [principledMaterial, principledMaterial]
                        }
                    }

                    NumberAnimation {
                        property real phase: 0
                        id: numberAnimation
                        target: numberAnimation
                        property: "phase"
                        duration: 2500
                        running: true
                        loops: -1
                        to: 0
                        from: -360
                    }
                }

                FlashEffect {
                    id: flash
                    visible: demoUI.mainSelection == "Flash"
                }

                ExplosionEffect {
                    id: explosion
                    visible: demoUI.mainSelection == "Explosion"

                    Connections {
                        target: explosion.timerVar
                        onTriggered: shockAnimator.restart()
                    }
                }

                BubblesEffect {
                    id: bubbles
                    x: 0
                    y: 60
                    visible: demoUI.mainSelection == "Bubbles"
                    z: 0

                    staticFlags: 0
                }

                CloudEffect {
                    id: cloud
                    y: 100
                    visible: demoUI.mainSelection == "Cloud"
                    staticFlags: 0
                }

                LineparticleEffect {
                    id: lineparticleEffect
                    visible: demoUI.mainSelection == "Light Trails"
                }

                AmbientEffect {
                    id: ambientparticles
                    visible: demoUI.mainSelection == "Particles"
                }

                HeatwaveEffect {
                    id: heatwave
                    visible: demoUI.mainSelection == "Heatwave"
                    staticFlags: 0
                }

                RainEffect {
                    id: rain
                    y: -100
                    visible: demoUI.mainSelection == "Rain" //rainBtn.checked
                }

                SmokeEffect {
                    id: smoke
                    y: 50
                    visible: demoUI.mainSelection == "Smoke"
                    staticFlags: 0
                }

                SteamEffect {
                    id: steam
                    y: 30
                    visible: demoUI.mainSelection == "Steam"
                    z: 0
                }

                SnowEffect {
                    id: snow
                    visible: demoUI.mainSelection == "Snow"
                }

                FireEffect {
                    id: fire
                    y: 50
                    visible: demoUI.mainSelection == "Fire"
                    staticFlags: 0
                }

                ShockwaveEffect {
                    id: shockwave
                    y: 10
                    visible: demoUI.mainSelection == "Shockwave"
                    Connections {
                        target: shockwave.timerVar
                        onTriggered: shockAnimator.restart()
                    }
                }

                DustEffect {
                    id: dust
                    visible: demoUI.mainSelection == "Dust"

                    staticFlags: 0
                }
            }

            Zen_SceneTest {
                id: zen_SceneTest
                y: 0
                scale.z: 100
                scale.y: 100
                scale.x: 100
            }


            /*ReflectionProbe {
                id: reflectionProbe
                refreshMode: ReflectionProbe.EveryFrame
                debugView: true
                boxSize.z: 100000
                boxSize.y: 100000
                boxSize.x: 100000
            }*/
        }

        MouseRotatorAndWASD {
            id: mouseRotatorAndWASD
            anchors.top: parent.top
            anchors.fill: parent
            anchors.topMargin: 0
        }

        MouseArea {
            id: mouseArea
            anchors.fill: parent
            Connections {
                target: mouseArea

                onWheel: sceneCamera.fieldOfView += wheel.angleDelta.y * 0.04
                         * (sceneCamera.fieldOfView + wheel.angleDelta.y * 0.04 > 0.0)
                         * (sceneCamera.fieldOfView + wheel.angleDelta.y * 0.04 < 90.0)
            }
            PinchArea {
                id: pinchArea
                anchors.fill: parent

                Connections {
                    target: pinchArea
                    onPinchUpdated: if (pinch.previousScale - pinch.scale > 0.008
                                            || pinch.previousScale - pinch.scale < -1 * 0.008) {
                                        let velocity = (pinch.previousScale
                                                        - pinch.scale) > 0 ? 1.25 : -1.25
                                        sceneCamera.fieldOfView += velocity
                                                * (sceneCamera.fieldOfView + velocity > 0.0)
                                                * (sceneCamera.fieldOfView + velocity < 90.0)
                                    }
                }
            }
        }
    }

    Rectangle {
        id: rectangle
        width: 904
        height: 80
        visible: false
        color: "#ffffff"
        anchors.top: parent.top
        anchors.topMargin: 0
        anchors.horizontalCenter: parent.horizontalCenter

        RoundButton {
            id: rainBtn
            width: 120
            height: 64
            radius: checked ? 32 : 8
            text: "Rain"
            anchors.left: parent.left
            anchors.top: parent.top
            anchors.leftMargin: 8
            anchors.topMargin: 8
            checkable: true
            highlighted: true
        }

        RoundButton {
            id: snowBtn
            width: 120
            height: 64
            radius: checked ? 32 : 8
            text: "Snow"
            anchors.left: parent.left
            anchors.top: parent.top
            highlighted: true
            checkable: true
            anchors.topMargin: 8
            anchors.leftMargin: 136
        }

        RoundButton {
            id: smokeBtn
            width: 120
            height: 64
            radius: checked ? 32 : 8
            text: "Smoke"
            anchors.left: parent.left
            anchors.top: parent.top
            highlighted: true
            checkable: true
            anchors.topMargin: 8
            anchors.leftMargin: 264
        }

        RoundButton {
            id: heatwaveBtn
            width: 120
            height: 64
            radius: checked ? 32 : 8
            text: "Heatwave"
            anchors.left: parent.left
            anchors.top: parent.top
            highlighted: true
            checkable: true
            anchors.topMargin: 8
            anchors.leftMargin: 392
        }

        RoundButton {
            id: fireBtn
            width: 120
            height: 64
            radius: checked ? 32 : 8
            text: "Fire"
            anchors.left: parent.left
            anchors.top: parent.top
            highlighted: true
            anchors.topMargin: 8
            anchors.leftMargin: 520
            checkable: true
        }

        RoundButton {
            id: sparkBtn
            width: 120
            height: 64
            radius: checked ? 32 : 8
            text: "Sparks"
            anchors.left: parent.left
            anchors.top: parent.top
            highlighted: true
            anchors.topMargin: 8
            anchors.leftMargin: 648
            checkable: true
        }

        RoundButton {
            id: dustBtn
            width: 120
            height: 64
            radius: checked ? 32 : 8
            text: "Dust"
            anchors.left: parent.left
            anchors.top: parent.top
            highlighted: true
            anchors.topMargin: 8
            anchors.leftMargin: 776
            checkable: true
        }
    }

    Item {
        id: __materialLibrary__

        CustomMaterial {
            id: skyMaterial
            cullMode: Material.FrontFaceCulling
            shadingMode: CustomMaterial.Shaded
            fragmentShader: "shaders/dynamicsky.frag"
            property real turbidity: demoUI.altScene ? 2 : 1
            property real exposure: demoUI.altScene ? 0.0015 : 0.05
            property real brightness: directionalLight.brightness
            property color skyColor: directionalLight.ambientColor
            property bool stars: true
            property TextureInput starMap: TextureInput {
                texture: Texture {
                    source: "images/LDR_RGBA_0.png"
                    mipFilter: Texture.Linear
                    generateMipmaps: true
                }
            }
            vertexShader: "shaders/dynamicsky.vert"
            PropertyAnimation {
                target: skyMaterial
                property: "timeOfDay"
                duration: 2500
            }
        }

        DefaultMaterial {
            id: defaultMaterial
            cullMode: Material.NoCulling
            vertexColorsEnabled: true
            objectName: "Default Material"
            diffuseColor: "#4aee45"
        }

        PrincipledMaterial {
            id: principledMaterial
            roughness: 0.1
            metalness: 1
            baseColor: "#ffffff"
            objectName: "New Material"
        }
    }

    Rectangle {
        id: rectangle2
        width: 1032
        height: 80
        visible: false
        color: "#ffffff"
        anchors.bottom: parent.bottom
        anchors.bottomMargin: 0
        RoundButton {
            id: flashBtn
            width: 120
            height: 64
            radius: checked ? 32 : 8
            text: "Flash"
            anchors.left: parent.left
            anchors.top: parent.top
            anchors.topMargin: 8
            checkable: true
            anchors.leftMargin: 8
            highlighted: true
        }

        RoundButton {
            id: shockwaveBtn
            width: 120
            height: 64
            radius: checked ? 32 : 8
            text: "Shockwave"
            anchors.left: parent.left
            anchors.top: parent.top
            anchors.topMargin: 8
            checkable: true
            anchors.leftMargin: 136
            highlighted: true
        }

        RoundButton {
            id: explosionBtn
            width: 120
            height: 64
            radius: checked ? 32 : 8
            text: "Explosion"
            anchors.left: parent.left
            anchors.top: parent.top
            anchors.topMargin: 8
            checkable: true
            anchors.leftMargin: 264
            highlighted: true
        }

        RoundButton {
            id: bubblesBtn
            width: 120
            height: 64
            radius: checked ? 32 : 8
            text: "Bubbles"
            anchors.left: parent.left
            anchors.top: parent.top
            anchors.topMargin: 8
            checkable: true
            anchors.leftMargin: 392
            highlighted: true
        }

        RoundButton {
            id: cloudBtn
            width: 120
            height: 64
            radius: checked ? 32 : 8
            text: "Cloud"
            anchors.left: parent.left
            anchors.top: parent.top
            anchors.topMargin: 8
            checkable: true
            anchors.leftMargin: 520
            highlighted: true
        }

        RoundButton {
            id: steamBtn
            width: 120
            height: 64
            radius: checked ? 32 : 8
            text: "Steam"
            anchors.left: parent.left
            anchors.top: parent.top
            anchors.leftMargin: 648
            checkable: true
            anchors.topMargin: 8
            highlighted: true
        }

        RoundButton {
            id: ambientBtn
            x: 9
            y: 9
            width: 120
            height: 64
            radius: checked ? 32 : 8
            text: "Ambient"
            anchors.left: parent.left
            anchors.top: parent.top
            checkable: true
            anchors.leftMargin: 776
            anchors.topMargin: 8
            highlighted: true
        }

        RoundButton {
            id: lineBtn
            x: -2
            y: -2
            width: 120
            height: 64
            radius: checked ? 32 : 8
            text: "Lines"
            anchors.left: parent.left
            anchors.top: parent.top
            checkable: true
            anchors.leftMargin: 904
            anchors.topMargin: 8
            highlighted: true
        }
        anchors.horizontalCenter: parent.horizontalCenter
    }

    DemoUI {
        id: demoUI
        demoName: "Particle effect library"
        category1Icon: "DemoUI/Elements.png"
        category1Text: "Elements"
        category2Icon: "DemoUI/Ambient.png"
        category2Text: "Ambient"
        category3Icon: "DemoUI/Dynamic.png"
        category3Text: "Dynamic"
        sView: view3D
    }

    TimeSource {
        id: timeSource
        sourceView: view3D
    }
}

/*##^##
Designer {
    D{i:0}D{i:3;cameraSpeed3d:25;cameraSpeed3dMultiplier:1}D{i:63;cameraSpeed3d:25;cameraSpeed3dMultiplier:1}
}
##^##*/
